@@ -200,7 +200,7 @@ pub async fn step_payment_handler<C: Context + Send>(
     let mut co = maybe_co.unwrap();
 
     match co.state {
-        State::PaymentInProgress(_) => {
+        State::PaymentInProgress{ reserved_since: _ } => {
             let mut payment = maybe_payment.ok_or(new_not_found_error(
                 "the associated payment could not be found",
             ))?;
@@ -268,10 +268,10 @@ pub async fn release_handler<C: Context + Send>(
 
     match co.state {
         State::Shopping => {}
-        State::ItemsConfirmed(_) => {
+        State::ItemsConfirmed{ reserved_since: _ } => {
             co.release(ctx).await?;
         }
-        State::PaymentInProgress(_) => {
+        State::PaymentInProgress{ reserved_since: _ } => {
             let mut payment = maybe_payment.ok_or(new_not_found_error(
                 "the associated payment could not be found",
             ))?;
