@@ -9,7 +9,6 @@ use crate::context::Context;
 use crate::customer::{Address, Contact};
 use crate::error::{new_application_error, new_invalid_state_error, Error};
 use crate::internal_context::InternalContext;
-use crate::inventory::ReserveResult;
 use crate::invoice::Invoice;
 use crate::item::Item;
 use crate::money::{Currency, Money};
@@ -21,10 +20,11 @@ use crate::shipping::{Fulfillment, FulfillmentSelection, ShippingQuote};
 pub mod handlers;
 
 #[derive(Serialize, Deserialize, JsonSchema)]
+#[serde(tag = "name", rename_all = "snake_case")]
 pub enum State {
-    Shopping,                         // transitions to ItemsConfirmed
-    ItemsConfirmed(DateTime<Utc>),    // transitions to Shopping, PaymentInProgress or Completed
-    PaymentInProgress(DateTime<Utc>), // can only enter this state from ItemsConfirmed (before deadline expiry). not allowed to free items due to deadline expiry in this state, unless payment is cancelled. transitions to either Shopping (if payment is cancelled) or Completed
+    Shopping,                                            // transitions to ItemsConfirmed
+    ItemsConfirmed { reserved_since: DateTime<Utc> }, // transitions to Shopping, PaymentInProgress or Completed
+    PaymentInProgress { reserved_since: DateTime<Utc> }, // can only enter this state from ItemsConfirmed. not allowed to free items in this state, unless payment is cancelled. transitions to either Shopping (if payment is cancelled) or Completed
     Completed, // can only enter this state from ItemsConfirmed or PaymentInProgress. this is a terminal state
 }
 
@@ -113,7 +113,7 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
     ) -> Result<(), Error> {
         self.reset_shipping_options();
         match ctx.reserve_items(&self.items).await? {
-            ReserveResult::Success(deadline) => {
+            None => {
                 if let Some(address) = &self.shipping_address {
                     self.shipping_quotes = ctx
                         .get_shipping_quotes(
@@ -127,12 +127,12 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
 
                 self.update_item_prices(ctx).await?;
                 self.update_invoice(ctx).await?;
-                self.state = State::ItemsConfirmed(deadline);
+                self.state = State::ItemsConfirmed {
+                    reserved_since: Utc::now(),
+                };
                 Ok(())
             }
-            ReserveResult::StockIssue(stock_issues) => {
-                Err(new_application_error("STOCK_ISSUE", stock_issues))
-            }
+            Some(stock_issues) => Err(new_application_error("STOCK_ISSUE", stock_issues)),
         }
     }
 
@@ -140,14 +140,14 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
         &mut self,
         ctx: &mut C,
         args: <C as PaymentProcessor>::InitArgs,
-        deadline: DateTime<Utc>,
+        reserved_since: DateTime<Utc>,
     ) -> Result<(), Error> {
         let charge_amount = self.invoice.as_ref().unwrap().initial_charge_amount.clone();
         let payment =
             payment_handlers::create_handler(ctx, Initiator::Checkout(self), charge_amount, args)
                 .await?;
         self.payment_id = Some(payment.id.clone());
-        self.state = State::PaymentInProgress(deadline);
+        self.state = State::PaymentInProgress { reserved_since };
 
         Ok(())
     }
@@ -162,7 +162,7 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
     async fn ensure_shopping_state<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         match self.state {
             State::Shopping => Ok(()),
-            State::ItemsConfirmed(_) => self.enter_shopping_state(ctx).await,
+            State::ItemsConfirmed { reserved_since: _ } => self.enter_shopping_state(ctx).await,
             _ => Err(new_invalid_state_error(
                 "must be in the Shopping or ItemsConfirmed state",
             )),
@@ -174,7 +174,7 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
         ctx: &mut C,
     ) -> Result<(), Error> {
         match self.state {
-            State::ItemsConfirmed(_) => Ok(()),
+            State::ItemsConfirmed { reserved_since: _ } => Ok(()),
             State::Shopping => self.enter_items_confirmed_state(ctx).await,
             _ => Err(new_invalid_state_error(
                 "must be in the Shopping or ItemsConfirmed state",
@@ -188,8 +188,8 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
         args: <C as PaymentProcessor>::InitArgs,
     ) -> Result<(), Error> {
         match self.state {
-            State::PaymentInProgress(_) => Ok(()),
-            State::ItemsConfirmed(deadline) => {
+            State::PaymentInProgress { reserved_since: _ } => Ok(()),
+            State::ItemsConfirmed { reserved_since } => {
                 // check for contact information
                 if self.contact.is_none() {
                     return Err(new_application_error(
@@ -206,7 +206,7 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
                     ));
                 }
 
-                self.enter_payment_in_progress_state(ctx, args, deadline)
+                self.enter_payment_in_progress_state(ctx, args, reserved_since)
                     .await
             }
             _ => Err(new_invalid_state_error(
@@ -218,7 +218,7 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
     async fn ensure_completed_state(&mut self) -> Result<(), Error> {
         match self.state {
             State::Completed => Ok(()),
-            State::ItemsConfirmed(_) => {
+            State::ItemsConfirmed { reserved_since: _ } => {
                 // check for contact information
                 if self.contact.is_none() {
                     return Err(new_application_error(
@@ -237,18 +237,18 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
 
                 self.enter_completed_state().await
             }
-            State::PaymentInProgress(_) => self.enter_completed_state().await,
+            State::PaymentInProgress { reserved_since: _ } => self.enter_completed_state().await,
             _ => Err(new_invalid_state_error(
                 "must be in the PaymentInProgress or Completed state",
             )),
         }
     }
 
-    async fn ensure_not_completed_state(&mut self) -> Result<(), Error> {
+    async fn ensure_not_payment_in_progress_or_completed_state(&mut self) -> Result<(), Error> {
         match self.state {
-            State::Completed => Err(new_invalid_state_error(
-                "should be in any state other than Completed",
-            )),
+            State::PaymentInProgress { reserved_since: _ } | State::Completed => Err(
+                new_invalid_state_error("should be in any state other than PaymentInProgress or Completed"),
+            ),
             _ => Ok(()),
         }
     }
@@ -363,7 +363,8 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
         ctx: &mut C,
         contact: Contact,
     ) -> Result<(), Error> {
-        self.ensure_not_completed_state().await?;
+        self.ensure_not_payment_in_progress_or_completed_state()
+            .await?;
 
         self.contact = Some(contact);
 
@@ -375,11 +376,12 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
         ctx: &mut C,
         address: Address,
     ) -> Result<(), Error> {
-        self.ensure_not_completed_state().await?;
+        self.ensure_not_payment_in_progress_or_completed_state()
+            .await?;
 
         self.reset_shipping_options();
         self.shipping_address = Some(address);
-        if let State::ItemsConfirmed(_) = self.state {
+        if let State::ItemsConfirmed { reserved_since: _ } = self.state {
             self.shipping_quotes = ctx
                 .get_shipping_quotes(
                     &self.currency,
@@ -446,25 +448,32 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
         ctx: &mut C,
         payment_args: <C as PaymentProcessor>::InitArgs,
     ) -> Result<(), Error> {
-        if !matches!(self.state, State::ItemsConfirmed(_)) {
+        if let State::ItemsConfirmed { reserved_since } = self.state {
+            self.ensure_completed_state().await?;
+            let order = Order::new_from_checkout_missing_payment(
+                internal_ctx,
+                ctx,
+                self,
+                reserved_since,
+                payment_args,
+            )
+            .await?;
+            self.order_id = Some(order.id.clone());
+
+            return ctx.update_checkout(self).await;
+        } else {
             return Err(new_invalid_state_error(
                 "must be in the ItemsConfirmed state",
             ));
         }
-
-        self.ensure_completed_state().await?;
-        let order =
-            Order::new_from_checkout_missing_payment(internal_ctx, ctx, self, payment_args).await?;
-        self.order_id = Some(order.id.clone());
-
-        ctx.update_checkout(self).await
     }
 
     pub async fn release<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         match self.state {
             State::Shopping => Ok(()),
-            State::ItemsConfirmed(_) | State::PaymentInProgress(_) => {
-                if let State::PaymentInProgress(_) = self.state {
+            State::ItemsConfirmed { reserved_since: _ }
+            | State::PaymentInProgress { reserved_since: _ } => {
+                if let State::PaymentInProgress { reserved_since: _ } = self.state {
                     self.payment_id = None;
                 }
 
@@ -485,7 +494,7 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
         internal_ctx: &InternalContext,
         ctx: &mut C,
     ) -> Result<(), Error> {
-        if !matches!(self.state, State::PaymentInProgress(_)) {
+        if !matches!(self.state, State::PaymentInProgress { reserved_since: _ }) {
             return Err(new_invalid_state_error(
                 "must be in the PaymentInProgress state",
             ));
