@@ -1,5 +1,4 @@
 use async_trait::async_trait;
-use chrono::{DateTime, Duration, Utc};
 use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
 
@@ -18,16 +17,22 @@ pub struct StockUnit {
     quantity: u64,
 }
 
-pub enum ReserveResult {
-    Success(DateTime<Utc>),
-    StockIssue(Vec<StockIssue>),
+impl From<&Item> for StockUnit {
+    fn from(item: &Item) -> Self {
+        Self {
+            sku: item.sku.clone(),
+            quantity: item.quantity,
+        }
+    }
 }
 
 #[async_trait]
 pub trait InventoryController {
-    async fn reserve_items(&mut self, _items: &Vec<Item>) -> Result<ReserveResult, Error> {
-        let deadline = Utc::now() + Duration::days(365);
-        Ok(ReserveResult::Success(deadline))
+    async fn reserve_items(
+        &mut self,
+        _items: &Vec<Item>,
+    ) -> Result<Option<Vec<StockIssue>>, Error> {
+        Ok(None)
     }
 
     async fn free_items(&mut self, _items: &Vec<Item>) -> Result<(), Error> {
