@@ -53,7 +53,7 @@ pub async fn step_payment_handler<C: Context + Send>(
     let mut od = maybe_od.unwrap();
 
     match od.state {
-        OrderState::PaymentInProgress => {
+        OrderState::PaymentInProgress { reserved_since: _ } => {
             let mut payment = maybe_payment.ok_or(new_not_found_error(
                 "the associated payment could not be found",
             ))?;
@@ -138,7 +138,7 @@ pub async fn cancel_handler<C: Context + Send>(
 
     match od.state {
         OrderState::Cancelled => {}
-        OrderState::PaymentInProgress => {
+        OrderState::PaymentInProgress { reserved_since: _ } => {
             let mut payment = maybe_payment.ok_or(new_not_found_error(
                 "the associated payment could not be found",
             ))?;
