@@ -1,4 +1,5 @@
 use async_trait::async_trait;
+use chrono::{DateTime, Utc};
 use cloud_pubsub;
 use schemars::JsonSchema;
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
@@ -11,6 +12,7 @@ use crate::{
     customer::{Address, Contact},
     error::{new_application_error, new_invalid_state_error, Error},
     internal_context::InternalContext,
+    inventory::StockUnit,
     invoice::Invoice,
     item::Item,
     payment::{handlers as payment_handlers, Initiator},
@@ -36,12 +38,13 @@ pub struct Order<P> {
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
+#[serde(tag = "name", rename_all = "snake_case")]
 pub enum OrderState {
-    PaymentInProgress, // initial state if the order is created via a checkout without payment; transitions to PendingConfirmation or Cancelled
+    PaymentInProgress { reserved_since: DateTime<Utc> }, // initial state if the order is created via a checkout without payment; transitions to PendingConfirmation or Cancelled
     PendingConfirmation, // initial state if the order is created via a checkout with payment; transitions to Confirmed or Cancelled
     Confirmed, // can only enter this state from PendingConfirmation; transitions to Fulfilled or Cancelled
-    Fulfilled, // a terminal state
-    Cancelled, // a terminal state
+    Fulfilled, // a terminal state. can only enter this state from Confirmed
+    Cancelled, // a terminal state. can enter this state from every other non-terminal state
 }
 
 #[derive(Debug, Serialize, Deserialize, JsonSchema)]
@@ -152,12 +155,13 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
         internal_ctx: &InternalContext,
         ctx: &mut C,
         checkout: &Checkout<P>,
+        reserved_since: DateTime<Utc>,
         payment_args: <C as PaymentProcessor>::InitArgs,
     ) -> Result<Self, Error> {
         let mut order = Order {
             id: Uuid::new_v4().to_string(),
             state_reconciled: false,
-            state: OrderState::PaymentInProgress,
+            state: OrderState::PaymentInProgress { reserved_since },
             contact: checkout.contact.as_ref().unwrap().clone(),
             shipping_address: checkout.shipping_address.as_ref().unwrap().clone(),
             items: checkout.items.clone(),
@@ -220,7 +224,7 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
     ) -> Result<(), Error> {
         match self.state {
             OrderState::PendingConfirmation => Ok(()),
-            OrderState::PaymentInProgress => {
+            OrderState::PaymentInProgress { reserved_since: _ } => {
                 self.transition_state(internal_ctx, OrderState::PendingConfirmation)
                     .await
             }
@@ -266,7 +270,7 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
     ) -> Result<(), Error> {
         match self.state {
             OrderState::Cancelled => Ok(()),
-            OrderState::PaymentInProgress
+            OrderState::PaymentInProgress { reserved_since: _ }
             | OrderState::PendingConfirmation
             | OrderState::Confirmed => {
                 self.transition_state(internal_ctx, OrderState::Cancelled)
@@ -319,6 +323,14 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
         ctx: &mut C,
     ) -> Result<(), Error> {
         self.ensure_fulfilled_state(internal_ctx).await?;
+
+        // free reserved items
+        ctx.free_items(&self.items).await?;
+
+        // remove stock
+        let stock: Vec<StockUnit> = self.items.iter().map(|item| item.into()).collect();
+        ctx.remove_stock(&stock).await?;
+
         ctx.update_order(self).await
     }
 
@@ -328,6 +340,10 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
         ctx: &mut C,
     ) -> Result<(), Error> {
         self.ensure_cancelled_state(internal_ctx).await?;
+
+        // free reserved items
+        ctx.free_items(&self.items).await?;
+
         ctx.update_order(self).await
     }
 }
@@ -343,7 +359,9 @@ async fn reconcile<C: Context + Send>(
     order.populate_associations(ctx).await?;
 
     match order.state {
-        OrderState::PaymentInProgress => ctx.on_payment_in_progress(order).await?,
+        OrderState::PaymentInProgress { reserved_since: _ } => {
+            ctx.on_payment_in_progress(order).await?
+        }
         OrderState::PendingConfirmation => ctx.on_pending_confirmation(order).await?,
         OrderState::Confirmed => ctx.on_confirmed(order).await?,
         OrderState::Fulfilled => ctx.on_fulfilled(order).await?,
